@@ -1,4 +1,4 @@
-/* $Id: calendar-conduit.c,v 1.20 1999/12/03 00:14:36 eskil Exp $ */
+/* $Id: calendar-conduit.c,v 1.23 2000/01/05 12:12:44 eskil Exp $ */
 
 #include <glib.h>
 #include <gnome.h>
@@ -43,14 +43,14 @@ GnomePilotConduit * conduit_get_gpilot_conduit (guint32);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit*);
 void local_record_from_icalobject(GCalLocalRecord *local,iCalObject *obj); 
 
-#define CONDUIT_VERSION "0.8.6"
+#define CONDUIT_VERSION "0.8.11"
 #ifdef G_LOG_DOMAIN
 #undef G_LOG_DOMAIN
 #endif
 #define G_LOG_DOMAIN "gcalconduit" 
 
 #define DEBUG_CALCONDUIT 
-#undef DEBUG_CALCONDUIT 
+#undef DEBUG_CALCONDUIT
 
 #ifdef DEBUG_CALCONDUIT
 #define show_exception(e) g_warning ("Exception: %s\n", CORBA_exception_id (e))
@@ -63,6 +63,14 @@ void local_record_from_icalobject(GCalLocalRecord *local,iCalObject *obj);
 #define WARN(e...) g_log(G_LOG_DOMAIN,G_LOG_LEVEL_WARNING, e)
 #define INFO(e...) g_log(G_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE, e)
 
+#define catch_ret_val(_env,ret)                                                                 \
+  if (_env._major != CORBA_NO_EXCEPTION) {                                                      \
+        g_log(G_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,"%s:%d: Caught exception",__FILE__,__LINE__);    \
+        g_warning ("Exception: %s\n", CORBA_exception_id (&(_env)));                               \
+	CORBA_exception_free(&(_env));                                                             \
+	return ret;                                                                             \
+  }
+
 static int
 start_calendar_server (GnomePilotConduitStandardAbs *conduit,
 		       GCalConduitContext *ctxt) 
@@ -75,7 +83,7 @@ start_calendar_server (GnomePilotConduitStandardAbs *conduit,
 						       "IDL:GNOME:Calendar:Repository:1.0",
 						       0, NULL);
 	if (ctxt->calendar == CORBA_OBJECT_NIL) {
-		g_error ("Can not communicate with GnomeCalendar server");
+		g_warning ("Can not communicate with GnomeCalendar server");
 		return -1;
 	}
   
@@ -137,56 +145,6 @@ get_calendar_objects(GnomePilotConduitStandardAbs *conduit,
 	return result;
 }
 
-#if 0
-static GList * 
-get_calendar_objects(GnomePilotConduitStandardAbs *conduit) 
-{
-	char *vcalendar_string;
-	char *error;
-	GList *retval,*l;
-	Calendar *cal;
-
-	g_return_val_if_fail(conduit!=NULL,NULL);
-
-	vcalendar_string = 
-		GNOME_Calendar_Repository_get_objects (calendar, &ev);
-
-	cal = calendar_new("Temporary");
-
-	error = calendar_load_from_memory(cal,vcalendar_string);
-
-  	if (ev._major == CORBA_USER_EXCEPTION){
-		INFO ("Object did not exist");
-		show_exception(&ev);
-		CORBA_exception_free(&ev); 
-		return;
-	} else if(ev._major != CORBA_NO_EXCEPTION) {
-		WARN (_("Error while communicating with calendar server"));
-		show_exception(&ev);
-		CORBA_exception_free(&ev); 
-		return;
-	} 
-
-	if(error != NULL) {
-		WARN ("Error while converting records");
-		WARN ("Error : %s",error);
-		return NULL;
-	}
-	retval = NULL;
-	for(l=cal->events ; l ; l=l->next) {
-		LOG ("duping %d [%s]",
-			((iCalObject*)l->data)->pilot_id,
-			((iCalObject*)l->data)->summary);
-		retval = g_list_prepend(retval,ical_object_duplicate(l->data));
-	}
-
-	/* g_free(vcalendar_string); FIXME: this coredumps, but won't it leak without ? */
-	calendar_destroy(cal);
-
-	return retval;  
-}
-#endif
-
 static void 
 local_record_from_ical_uid(GCalLocalRecord *local,
 			   char *uid,
@@ -577,6 +535,31 @@ update_record (GnomePilotConduitStandardAbs *conduit,
 	return 0;
 }
 
+static void
+check_for_slow_setting(GnomePilotConduit *c, 
+		       GCalConduitContext *ctxt)
+{
+	CORBA_long entry_number;
+	entry_number = 
+		GNOME_Calendar_Repository_get_number_of_objects(ctxt->calendar, 
+								GNOME_Calendar_Repository_ANY,
+								&(ctxt->ev));
+
+	if (ctxt->ev._major == CORBA_USER_EXCEPTION){
+		show_exception(&(ctxt->ev));
+		CORBA_exception_free(&(ctxt->ev)); 
+	} else if(ctxt->ev._major != CORBA_NO_EXCEPTION) {
+	        WARN (_("Error while communicating with calendar server"));
+		show_exception(&(ctxt->ev));
+		CORBA_exception_free(&(ctxt->ev)); 
+	} else {
+		LOG (_("Calendar holds %d entries"),entry_number);
+		/* If the local base is empty, do a slow sync */
+		if ( entry_number <= 0) {
+			gnome_pilot_conduit_standard_set_slow(GNOME_PILOT_CONDUIT_STANDARD(c));
+		}
+	}
+}
 
 static gint
 pre_sync(GnomePilotConduit *c, 
@@ -584,29 +567,55 @@ pre_sync(GnomePilotConduit *c,
 	 GCalConduitContext *ctxt) 
 {
 	int l;
+	gint num_records;
 	unsigned char *buf;
   
 	g_message ("GnomeCal Conduit v.%s",CONDUIT_VERSION);
 
 	ctxt->calendar = CORBA_OBJECT_NIL;
 	
-	if (start_calendar_server(GNOME_PILOT_CONDUIT_STANDARD_ABS(c),GET_GCALCONTEXT(c)) != 0) {
+	if (start_calendar_server(GNOME_PILOT_CONDUIT_STANDARD_ABS(c),ctxt) != 0) {
 		WARN(_("Could not start gnomecal server"));
+		gnome_pilot_conduit_error(GNOME_PILOT_CONDUIT(c),
+					  _("Could not start gnomecal server"));
 		return -1;
 	}
   
+	/* Set the counters for the progress bar crap */
+	num_records = GNOME_Calendar_Repository_get_number_of_objects (ctxt->calendar, GNOME_Calendar_Repository_ANY, &(ctxt->ev));
+	catch_ret_val(ctxt->ev,-1);
+	gnome_pilot_conduit_standard_abs_set_num_local_records(GNOME_PILOT_CONDUIT_STANDARD_ABS(c),
+							       num_records);
+	num_records = GNOME_Calendar_Repository_get_number_of_objects (ctxt->calendar, GNOME_Calendar_Repository_MODIFIED, &(ctxt->ev));
+	catch_ret_val(ctxt->ev,-1);
+	gnome_pilot_conduit_standard_abs_set_num_updated_local_records(GNOME_PILOT_CONDUIT_STANDARD_ABS(c),
+								       num_records);
+	num_records = GNOME_Calendar_Repository_get_number_of_objects (ctxt->calendar, GNOME_Calendar_Repository_NEW, &(ctxt->ev));
+	catch_ret_val(ctxt->ev,-1);
+	gnome_pilot_conduit_standard_abs_set_num_new_local_records(GNOME_PILOT_CONDUIT_STANDARD_ABS(c),
+								   num_records);
+	num_records = GNOME_Calendar_Repository_get_number_of_objects (ctxt->calendar, GNOME_Calendar_Repository_DELETED, &(ctxt->ev));
+	catch_ret_val(ctxt->ev,-1);
+	gnome_pilot_conduit_standard_abs_set_num_deleted_local_records(GNOME_PILOT_CONDUIT_STANDARD_ABS(c),
+								       num_records);
+
 	gtk_object_set_data(GTK_OBJECT(c),"dbinfo",dbi);
   
 	/* load_records(c); */
 
 	buf = (unsigned char*)g_malloc(0xffff);
-	if((l=dlp_ReadAppBlock(dbi->pilot_socket,dbi->db_handle,0,(unsigned char *)buf,0xffff))<0) {
-		WARN(_("Could not read pilot's Appoint application block"));
+	if((l=dlp_ReadAppBlock(dbi->pilot_socket,dbi->db_handle,0,(unsigned char *)buf,0xffff)) < 0) {
+		WARN(_("Could not read pilot's DateBook application block"));
+		WARN("dlp_ReadAppBlock(...) = %d",l);
+		gnome_pilot_conduit_error(GNOME_PILOT_CONDUIT(c),
+			     _("Could not read pilot's DateBook application block"));
 		return -1;
 	}
 	unpack_AppointmentAppInfo(&(ctxt->ai),buf,l);
 	g_free(buf);
 
+	check_for_slow_setting(c,ctxt);
+
 	return 0;
 }
 
@@ -1360,9 +1369,7 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	gtk_object_set_data(retval,"gcalconduit_cfg",cfg);
 
 	gcalconduit_new_context(&ctxt,cfg);
-	/* No real need to set it, since all signal are given this
-	   as their user data */
-	gtk_object_set_data(retval,"gcalconduit_context",ctxt);
+	gtk_object_set_data(GTK_OBJECT(retval),"gcalconduit_context",ctxt);
 
 	gtk_signal_connect (retval, "match_record", (GtkSignalFunc) match_record, ctxt);
 	gtk_signal_connect (retval, "free_match", (GtkSignalFunc) free_match, ctxt);
@@ -1383,8 +1390,6 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	gtk_signal_connect (retval, "transmit", (GtkSignalFunc) transmit, ctxt);
 	gtk_signal_connect (retval, "pre_sync", (GtkSignalFunc) pre_sync, ctxt);
 
-
-
 	return GNOME_PILOT_CONDUIT (retval);
 }
 
