@@ -19,7 +19,7 @@
 
 #include "address-conduit.h"
 
-#define CONDUIT_VERSION  "0.2"
+#define CONDUIT_VERSION  "0.3"
 
 /************************************
   Address field index:
@@ -173,17 +173,31 @@ iterate_foreach(GCardLocalRecord *local,IterateData *d)
 	}
 }
 
-/**
-   safe strdup. Exactly like g_strdup, but for later
-   use with free() instead of g_free(). Needed fore free_Address
- */
-static char * 
-entry_strdup(char* s)
+static gboolean
+need_quote (const gchar *str)
 {
-	if(s) return strdup(s);
-	else return 0;
+	/* TODO: Are there any other characters that will require quoting? */
+	if(!str) return FALSE;
+	if(strchr(str,'\n') != NULL)
+		return TRUE;
+	return FALSE;
 }
-  
+
+
+static gchar * 
+str_card_to_addr(gchar* s)
+{
+        /* Must use strdup and not g_strdup because, free_Address uses free()*/
+	if(s) return strdup(s);
+	return 0;
+}
+
+static gchar *
+str_addr_to_card(gchar* s)
+{
+	return g_strdup(s);
+}
+
 static void 
 load_foreach(Card * card, GnomePilotConduitStandardAbs *abs) 
 {
@@ -197,44 +211,40 @@ load_foreach(Card * card, GnomePilotConduitStandardAbs *abs)
 	local->gcard=card;
 	
 	/* create Address from GCard 
-	   NOTE: VERY IMPORTANT to use entry_strdup and not g_strdup for Address entries,
-	   because we'll use free_Address later which does a simple free()
 	*/
 	local->addr=g_malloc0(sizeof(struct Address));
 	
 	if(card->name.prop.used) {
-		local->addr->entry[0]=entry_strdup(card->name.family);
-		local->addr->entry[1]=entry_strdup(card->name.given);
+		local->addr->entry[0]=str_card_to_addr(card->name.family);
+		local->addr->entry[1]=str_card_to_addr(card->name.given);
 	}
 	
 	if(card->org.prop.used) {
-		local->addr->entry[2]=entry_strdup(card->org.name);
+		local->addr->entry[2]=str_card_to_addr(card->org.name);
 	}
 	/* phones & email */
 	/* TODO: What's the good algorithm if we have more than 5 combined email
 	   & phones? */
 	count=0;
-	if(card->phone.prop.used &&
-	   card->phone.l != NULL) {
+	if(card->phone.l != NULL) {
 		GList* tmp = card->phone.l;
 		while (tmp != NULL && count<5) {
 			CardPhone *phone=(CardPhone*)tmp->data;
 			if(phone->prop.used){
 				local->addr->phoneLabel[count]=convert_phone_type(phone->type);
-				local->addr->entry[3+count]=entry_strdup(phone->data);
+				local->addr->entry[3+count]=str_card_to_addr(phone->data);
 			}
 			tmp = g_list_next(tmp);
 			count++;
 		}
 	}
-	if(card->email.prop.used &&
-	   card->email.l != NULL) {
+	if(card->email.l != NULL) {
 		GList* tmp = card->email.l;
 		while (tmp != NULL && count<5) {
 			CardEMail *email=(CardEMail*)tmp->data;
 			if(email->prop.used){
 				local->addr->phoneLabel[count]=EMAIL_INDEX;
-				local->addr->entry[3+count]=entry_strdup(email->data);
+				local->addr->entry[3+count]=str_card_to_addr(email->data);
 			}
 			tmp = g_list_next(tmp);
 			count++;
@@ -242,23 +252,22 @@ load_foreach(Card * card, GnomePilotConduitStandardAbs *abs)
 	}
 	
 	/*address, just take the first one. */
-	if(card->deladdr.prop.used &&
-	   card->deladdr.l != NULL) {
+	if(card->deladdr.l != NULL) {
 		CardDelAddr *address=card->deladdr.l->data;
-		local->addr->entry[8]=entry_strdup(address->data[STREET]);
-		local->addr->entry[9]=entry_strdup(address->data[CITY]);
-		local->addr->entry[10]=entry_strdup(address->data[REGION]);
-		local->addr->entry[11]=entry_strdup(address->data[CODE]);
-		local->addr->entry[12]=entry_strdup(address->data[COUNTRY]);
+		local->addr->entry[8]=str_card_to_addr(address->data[STREET]);
+		local->addr->entry[9]=str_card_to_addr(address->data[CITY]);
+		local->addr->entry[10]=str_card_to_addr(address->data[REGION]);
+		local->addr->entry[11]=str_card_to_addr(address->data[CODE]);
+		local->addr->entry[12]=str_card_to_addr(address->data[COUNTRY]);
 	}
 
 	if(card->title.prop.used) {
-		local->addr->entry[13]=entry_strdup(card->title.str);
+		local->addr->entry[13]=str_card_to_addr(card->title.str);
 	}
 	/* TODO: Deal with custom fields somehow */
 
 	if(card->comment.prop.used) {
-		local->addr->entry[18]=entry_strdup(card->comment.str);
+		local->addr->entry[18]=strdup(card->comment.str);
 	}
 
 	if(card->uid.prop.used) {
@@ -349,14 +358,14 @@ pre_sync(GnomePilotConduit *c, GnomePilotDBInfo *dbi)
 	unsigned char *buf;
 
 	g_message("GnomeCard conduit v %s",CONDUIT_VERSION);
-	LOG("PreSync",1);
+	LOG("PreSync");
 
 	GET_CONDUIT_DATA(c)->dbi=dbi;
   
 	load_records((GnomePilotConduitStandardAbs*)c); 
 	buf = (unsigned char*)g_malloc(0xffff);
 	if((l=dlp_ReadAppBlock(dbi->pilot_socket,dbi->db_handle,0,(unsigned char *)buf,0xffff))<0) {
-		LOG("dlp_ReadAppBlock(...) failed",1);
+		LOG("dlp_ReadAppBlock(...) failed");
 		g_free(buf);
 		return -1;
 	}
@@ -378,7 +387,7 @@ match_record	(GnomePilotConduitStandardAbs *abs,
 		 gpointer data)
 {
 	GList *tmp;
-	LOG("MatchRecord",1);
+	LOG("MatchRecord");
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
 
@@ -396,7 +405,7 @@ free_match	(GnomePilotConduitStandardAbs *abs,
 		 GCardLocalRecord **local,
 		 gpointer data)
 {
-	LOG("FreeMatch",1);
+	LOG("FreeMatch");
 
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(*local!=NULL,-1);
@@ -410,7 +419,7 @@ archive_local (GnomePilotConduitStandardAbs *abs,
 	       GCardLocalRecord *local,
 	       gpointer data)
 {
-	LOG("ArchiveLocal",1);
+	LOG("ArchiveLocal");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.archived = 1; 
 	return 0;
@@ -422,7 +431,7 @@ archive_remote (GnomePilotConduitStandardAbs *abs,
 		PilotRecord *remote,
 		gpointer data)
 {
-	LOG("ArchiveRemote",1);
+	LOG("ArchiveRemote");
 	g_return_val_if_fail(remote!=NULL,-1);
 
 	if(local!=NULL)  local->local.archived = 1; 
@@ -437,16 +446,6 @@ archive_remote (GnomePilotConduitStandardAbs *abs,
 	return 0;
 }
 
-static gboolean
-need_quote (const gchar *str)
-{
-	/* TODO: Are there any other characters that will require quoting? */
-	if(!str) return FALSE;
-	if(strchr(str,'\n') != NULL)
-		return TRUE;
-	return FALSE;
-}
-
 static gint
 store_remote (GnomePilotConduitStandardAbs *abs,
 	      PilotRecord *remote,
@@ -461,7 +460,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 	int i;
 	gboolean present;
 
-	LOG("StoreRemote",1);
+	LOG("StoreRemote");
 	
 	g_return_val_if_fail(remote!=NULL,-1);
 	cd = GET_CONDUIT_DATA(abs);
@@ -495,8 +494,8 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 	if(local->addr->entry[0] != NULL ||
 	   local->addr->entry[1] != NULL) {
 		present=TRUE;
-		local->gcard->name.family=g_strdup(local->addr->entry[0]);
-		local->gcard->name.given=g_strdup(local->addr->entry[1]);
+		local->gcard->name.family=str_addr_to_card(local->addr->entry[0]);
+		local->gcard->name.given=str_addr_to_card(local->addr->entry[1]);
 		local->gcard->name.prop.used=TRUE;
 		if(need_quote(local->addr->entry[0]) ||
 		   need_quote(local->addr->entry[1]))
@@ -505,26 +504,27 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 	if(present) {
 		local->gcard->fname.prop.used=TRUE;
 		if(local->addr->entry[1]) {
-			local->gcard->fname.str=g_strjoin(" ",local->addr->entry[1],
-							  local->addr->entry[0],NULL);
+			gchar *s=g_strjoin(" ",local->addr->entry[1],local->addr->entry[0],NULL);
+			local->gcard->fname.str=str_addr_to_card(s);
+			g_free(s);
 			if(need_quote(local->addr->entry[0]) ||
 			   need_quote(local->addr->entry[1]))
 				local->gcard->fname.prop.encod=ENC_QUOTED_PRINTABLE;
 		} else {
-			local->gcard->fname.str=g_strdup(local->addr->entry[0]);
+			local->gcard->fname.str=str_addr_to_card(local->addr->entry[0]);
 			if(need_quote(local->addr->entry[0]))
 				local->gcard->fname.prop.encod=ENC_QUOTED_PRINTABLE;
 		}
 	}
 	if(local->addr->entry[2]) {
 		local->gcard->org.prop.used=TRUE;
-		local->gcard->org.name=g_strdup(local->addr->entry[2]);
+		local->gcard->org.name=str_addr_to_card(local->addr->entry[2]);
 		if(need_quote(local->addr->entry[2]))
 			local->gcard->org.prop.encod=ENC_QUOTED_PRINTABLE;
 		/* If name wasn't present use ORG as a Full name */
 		if(!present) {
 			local->gcard->fname.prop.used=TRUE;
-			local->gcard->fname.str=g_strdup(local->addr->entry[2]);		
+			local->gcard->fname.str=str_addr_to_card(local->addr->entry[2]);
 			if(need_quote(local->addr->entry[2]))
 				local->gcard->fname.prop.encod=ENC_QUOTED_PRINTABLE;
 		}
@@ -538,7 +538,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 				email->prop.used=TRUE;
 				email->prop.type=PROP_EMAIL;
 				email->type=EMAIL_INET;
-				email->data=g_strdup(local->addr->entry[i]);
+				email->data=str_addr_to_card(local->addr->entry[i]);
 				if(need_quote(local->addr->entry[i]))
 					email->prop.encod=ENC_QUOTED_PRINTABLE;
 				local->gcard->email.prop.used=TRUE;
@@ -549,7 +549,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 				phone->prop.used=TRUE;
 				phone->prop.type=PROP_PHONE;
 				phone->type=phone_type[local->addr->phoneLabel[i-3]];
-				phone->data=g_strdup(local->addr->entry[i]);
+				phone->data=str_addr_to_card(local->addr->entry[i]);
 				if(need_quote(local->addr->entry[i]))
 					phone->prop.encod=ENC_QUOTED_PRINTABLE;
 				local->gcard->phone.prop.used=TRUE;
@@ -571,11 +571,11 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 		address->prop.used=TRUE;		
 		address->prop.type=PROP_DELADDR;
 		address->type=ADDR_HOME;
-		address->data[STREET]=g_strdup(local->addr->entry[8]);
-		address->data[CITY]=g_strdup(local->addr->entry[9]);
-		address->data[REGION]=g_strdup(local->addr->entry[10]);
-		address->data[CODE]=g_strdup(local->addr->entry[11]);
-		address->data[COUNTRY]=g_strdup(local->addr->entry[12]);
+		address->data[STREET]=str_addr_to_card(local->addr->entry[8]);
+		address->data[CITY]=str_addr_to_card(local->addr->entry[9]);
+		address->data[REGION]=str_addr_to_card(local->addr->entry[10]);
+		address->data[CODE]=str_addr_to_card(local->addr->entry[11]);
+		address->data[COUNTRY]=str_addr_to_card(local->addr->entry[12]);
 		if(need_quote(local->addr->entry[8])||
 		   need_quote(local->addr->entry[9])||
 		   need_quote(local->addr->entry[10])||
@@ -588,7 +588,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 
 	if(local->addr->entry[13]) {
 		local->gcard->title.prop.used=TRUE;
-		local->gcard->title.str=g_strdup(local->addr->entry[13]);
+		local->gcard->title.str=str_addr_to_card(local->addr->entry[13]);
 		if(need_quote(local->addr->entry[13]))
 			local->gcard->title.prop.encod=ENC_QUOTED_PRINTABLE;
 	}
@@ -615,7 +615,7 @@ clear_status_archive_local (GnomePilotConduitStandardAbs *abs,
 			    GCardLocalRecord *local,
 			    gpointer data)
 {
-	LOG("ClearStatusArchiveLocal",1);
+	LOG("ClearStatusArchiveLocal");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.archived=0;
 	return 0;
@@ -626,7 +626,7 @@ iterate (GnomePilotConduitStandardAbs *abs,
 	 GCardLocalRecord **local,
 	 gpointer data)
 {
-	LOG("Iterate",1);
+	LOG("Iterate");
 	g_return_val_if_fail(local!=NULL,-1);
 
 	if(!*local) {
@@ -674,7 +674,7 @@ static gint
 purge (GnomePilotConduitStandardAbs *abs,
        gpointer data)
 {
-	LOG("Purge",1);
+	LOG("Purge");
 
 	spool_records(abs);
 	return 0;
@@ -686,7 +686,7 @@ set_status (GnomePilotConduitStandardAbs *abs,
 	    gint status,
 	    gpointer data)
 {
-	LOG("SetStatus",1);
+	LOG("SetStatus");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.attr = status; 
 	return 0;
@@ -698,7 +698,7 @@ set_archived (GnomePilotConduitStandardAbs *abs,
 	      gint archived,
 	      gpointer data)
 {
-	LOG("SetArchived",1);
+	LOG("SetArchived");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.archived = archived; 
 	return 0;
@@ -724,7 +724,7 @@ compare (GnomePilotConduitStandardAbs *abs,
 {
 	int i;
 	struct Address a;
-	LOG("Compare",1);
+	LOG("Compare");
 	
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
@@ -749,10 +749,16 @@ compare (GnomePilotConduitStandardAbs *abs,
 		}
 	}
 	for (i=0;i<19;i++) {
-		if(strcmp(a.entry[i],local->addr->entry[i])!=0) {
-			free_Address(&a);
-			return 1;
+		if(a.entry[i] == NULL && local->addr->entry[i]==NULL) {
+			continue;
 		}
+		if(a.entry[i] && local->addr->entry[i]) {
+			if(strcmp(a.entry[i],local->addr->entry[i])==0) {
+				continue;
+			}
+		}
+		free_Address(&a);
+		return 1;
 	}
 	free_Address(&a);
 	return 0;
@@ -764,7 +770,7 @@ compare_backup (GnomePilotConduitStandardAbs *abs,
 		PilotRecord *remote,
 		gpointer data)
 {
-	LOG("CompareBackup",1);
+	LOG("CompareBackup");
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
 
@@ -776,7 +782,7 @@ free_transmit (GnomePilotConduitStandardAbs *abs,
 	       PilotRecord **remote,
 	       gpointer data)
 {
-	LOG("FreeTransmit",1);
+	LOG("FreeTransmit");
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
 	g_return_val_if_fail(*remote!=NULL,-1);
@@ -790,7 +796,7 @@ static gint
 delete_all (GnomePilotConduitStandardAbs *abs,
 	    gpointer data)
 {
-	LOG("DeleteAll",1);
+	LOG("DeleteAll");
 	g_list_foreach(GET_CONDUIT_DATA(abs)->records,
 		       (GFunc)delete_records_foreach,
 		       NULL);
@@ -843,7 +849,7 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	gnome_pilot_conduit_construct(GNOME_PILOT_CONDUIT(retval),"address");
 
 	
-	LOG("creating GnomeCard conduit",0);
+	LOG("creating GnomeCard conduit");
 
 	g_assert (retval != NULL);
 	gtk_signal_connect (retval, "match_record", (GtkSignalFunc) match_record, NULL);
@@ -869,8 +875,9 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONFIG,cc);
 	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONDUIT,cd);
 	
-	if(cc->open_secret) gtk_object_set_data(GTK_OBJECT(retval),"db_open_mode",
-						GINT_TO_POINTER(dlpOpenReadWrite|dlpOpenSecret));
+	if(cc->open_secret) gnome_pilot_conduit_standard_abs_set_db_open_mode(GNOME_PILOT_CONDUIT(retval),
+									      dlpOpenReadWrite|dlpOpenSecret);
+
 	return GNOME_PILOT_CONDUIT (retval);
 }
 
@@ -879,7 +886,7 @@ conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit)
 {
 	ConduitData *cd=GET_CONDUIT_DATA(conduit);
 	ConduitCfg  *cfg=GET_CONDUIT_CFG(conduit);
-	LOG("destroying GnomeCard conduit",1);
+	LOG("destroying GnomeCard conduit");
 
 	g_list_foreach(cd->records,(GFunc)free_records_foreach,cd->records); 
 	g_list_free(cd->records);
