@@ -408,8 +408,8 @@ get_CardProperty(VObject *o)
 	
 	initPropIterator (&i, o);
 	while (moreIteration (&i)) {
-		VObject *vo = nextVObject (&i);
-		const char *n = vObjectName (vo);
+		VObject *vo, *cvo = nextVObject (&i);
+		const char *n = vObjectName (cvo);
 		int propid;
 	
 		propid = card_lookup_name(n);
@@ -418,18 +418,18 @@ get_CardProperty(VObject *o)
 			
 		 case PROP_VALUE:
 		
-			if (has (vo, VCContentIDProp))
+			if (has (cvo, VCContentIDProp))
 			  prop.value = VAL_CID;
-			else if (has (vo, VCURLValueProp))
+			else if (has (cvo, VCURLValueProp))
 			  prop.value = VAL_URL;
 			break;
 			
 		 case PROP_ENCODING:
-			if (has (vo, VCQuotedPrintableProp))
+			if (has (cvo, VCQuotedPrintableProp))
 			  prop.encod = ENC_QUOTED_PRINTABLE;
-			else if (has (vo, VC8bitProp))
+			else if (has (cvo, VC8bitProp))
 			  prop.encod = ENC_8BIT;
-			else if (has (vo, VCBase64Prop))
+			else if (has (cvo, VCBase64Prop))
 			  prop.encod = ENC_BASE64;
 			break;
 			
@@ -446,17 +446,17 @@ get_CardProperty(VObject *o)
 			break;
 	
 		 case PROP_LANG:
-			if (vObjectValueType(vo)) {
+			if (vObjectValueType(cvo)) {
 				prop.lang = 
-				  g_strdup(vObjectStringZValue (vo));
+				  g_strdup(vObjectStringZValue (cvo));
 			} else
 			  g_warning ("? < No value for LANG attribute.");
 			break;
 			
 		 case PROP_CHARSET:
-			if (vObjectValueType(vo)) {
+			if (vObjectValueType(cvo)) {
 				prop.charset = 
-				  g_strdup(vObjectStringZValue (vo));
+				  g_strdup(vObjectStringZValue (cvo));
 				g_warning(prop.charset); 
 			} else
 			  g_warning ("? < No value for CHARSET attribute.");
@@ -468,9 +468,9 @@ get_CardProperty(VObject *o)
 					c = malloc (sizeof (CardXAttribute));
 					c->name = g_strdup (n);
 					
-					if (vObjectValueType(vo))
+					if (vObjectValueType(cvo))
 					  c->data = 
-					  g_strdup(vObjectStringZValue (vo));
+					  g_strdup(vObjectStringZValue (cvo));
 					else
 					  c->data = NULL;
 					
@@ -786,9 +786,9 @@ strtoCardRev(char *str)
 	rev.utc = 0;
 	len = strlen(str);
 	
-	if (str[len] == 'Z') {              /* Is it UTC? */
+	if (str[len-1] == 'Z') {              /* Is it UTC? */
 		rev.utc = 1;
-		str[len] = 0;
+		str[len-1] = 0;
 	}
 	  
 	s[2] = 0;
@@ -796,7 +796,7 @@ strtoCardRev(char *str)
 	if (t) {                            /* Take the Time */
 		*t = 0;
 		t++;
-		if (strlen(t) > 2) {
+                if (strlen(t) >= 2) {
 			s[0] = t[0];
 			s[1] = t[1];
 			rev.tm.tm_hour = atoi(s);
@@ -809,7 +809,7 @@ strtoCardRev(char *str)
 		if (*t == ':')             /* Ignore ':' separator */
 		  t++;
 		
-		if (strlen(t) > 2) {
+		if (strlen(t) >= 2) {
 			s[0] = t[0];
 			s[1] = t[1];
 			rev.tm.tm_min = atoi(s);
@@ -822,7 +822,7 @@ strtoCardRev(char *str)
 		if (*t == ':')
 		  t++;
 		
-		if (strlen(t) > 2) {
+		if (strlen(t) >= 2) {
 			s[0] = t[0];
 			s[1] = t[1];
 			rev.tm.tm_sec = atoi(s);
@@ -831,7 +831,7 @@ strtoCardRev(char *str)
 			return rev;
 		}
 
-		if (strlen(str) > 3)
+		if (strlen(t) > 2)
 		  g_warning("? < Rev value is too long.");
 		
 	} else {
@@ -1161,6 +1161,7 @@ static VObject *
 add_CardProperty(VObject *o, CardProperty *prop)
 {
 	GList *node;
+	VObject *vo;
 	
 	switch (prop->encod) {
 	 case ENC_BASE64:
@@ -1197,7 +1198,7 @@ add_CardProperty(VObject *o, CardProperty *prop)
 		CardXAttribute *xa = (CardXAttribute *) node->data;
 		if (xa->data)
 		  addPropValue (o, xa->name, xa->data);
-		else
+		else if (! has (o,xa->name))
 		  addProp (o, xa->name);
 	}
 
